using System;
using System.Windows.Forms;
using Office = Microsoft.Office.Core;
using Word = Microsoft.Office.Interop.Word;
//using MSForms = Microsoft.Vbe.Interop.Forms;
using System.Xml;
using System.IO;

// Office integration attribute. Identifies the startup class for the document. Do not modify.
[assembly:System.ComponentModel.DescriptionAttribute("OfficeStartupClass, Version=1.0, Class=CustomerForm.OfficeCodeBehind")]

namespace CustomerForm
{
    /// <summary>
    /// Contains managed code extensions for the document.
    /// </summary>
    public class OfficeCodeBehind
    {

        /// <summary>
        /// Application object.
        /// </summary>
        internal Word.Application ThisApplication
        {
            get { return thisApplication;}
        }

        /// <summary>
        /// Document object.
        /// </summary>
        internal Word.Document ThisDocument
        {
            get { return thisDocument;}
        }
		// C#
		private object oMissing = System.Reflection.Missing.Value; 

        private Word.Application thisApplication = null;
        private Word.Document    thisDocument = null;

        private Word.DocumentEvents2_OpenEventHandler openEvent;
        private Word.DocumentEvents2_CloseEventHandler closeEvent;

		private Office.CommandBar cbar = null;
		private Office.CommandBarButton cbSaveAsNew = null;
		private Office.CommandBarButton cbEdit = null;
		private Office.CommandBarButton cbSave = null;
		private Office.CommandBarButton cbClear = null;
		private Office.CommandBarComboBox cbCustomerList = null;

		private Office._CommandBarButtonEvents_ClickEventHandler
			cbSaveAsNewClickEvent, cbEditClickEvent, cbSaveClickEvent,
			cbClearClickEvent;

		private Word.ApplicationEvents4_DocumentBeforeCloseEventHandler AppDocCloseEvent;
		private Word.ApplicationEvents4_DocumentBeforeSaveEventHandler AppDocSaveEvent;

        // Determines whether ActiveX controls will be set to
        // Run mode even if Word is in High security mode.
        // Set this variable to True if your solution uses
        // ActiveX controls.
        private bool toggleActiveXControls = false;
                                          
        #region Generated initialization code

        /// <summary>
        /// Default constructor.
        /// </summary>
        public OfficeCodeBehind()
        {
        }

        private bool designModeClose = false;

        /// <summary>
        /// Required procedure. Do not modify.
        /// </summary>
        /// <param name="application">Application object.</param>
        /// <param name="document">Document object.</param>
        public void _Startup(object application, object document)
        {
            this.thisApplication = application as Word.Application;
            this.thisDocument = document as Word.Document;

            openEvent = new Word.DocumentEvents2_OpenEventHandler (ThisDocument_Open);
            thisDocument.Open += openEvent;

            closeEvent = new Word.DocumentEvents2_CloseEventHandler(ThisDocument_Close);
            ((Word.DocumentEvents2_Event)thisDocument).Close += closeEvent;

            // Check if the document is in Design mode.
            if (ThisDocument.FormsDesign == true)
            {
                // Set ActiveX controls to Run mode.
                if (toggleActiveXControls == true)
                {
                    ThisDocument.ToggleFormsDesign();
                }

                // Check if the thisDocument is a document and not a template.
                if (thisDocument.Path != "")
                {
                    // Set the shutdown procedure to use design mode and then call the Open handler.
                    designModeClose = true;
                    ThisDocument_Open();

                    // Ensure that Open is not called twice.
                    thisDocument.Open -= openEvent;
                }
            }
        }

        /// <summary>
        /// Required procedure. Do not modify.
        /// </summary>
        public void _Shutdown()
        {
            // If the document is in design mode, call the Close handler.
            if (designModeClose == true)
                ThisDocument_Close();

            thisApplication = null;
            thisDocument = null;
        }

        /// <summary>
        /// Finds the control with the specified name in ThisDocument.
        /// </summary>
        /// <param name='name'>Name of the control to find.</param>
        /// <returns>
        /// Returns the specified control, or null if it is not found.
        /// </returns>
        object FindControl(string name)
        {
            return FindControl(name, ThisDocument);
        }
 
        /// <summary>
        /// Returns the control with the specified name in the specified document.
        /// </summary>
        /// <param name='name'>Name of the control to find.</param>
        /// <param name='document'>Document object that contains the control.</param>
        /// <returns>Returns the specified control, or null if it is not found.
        /// </returns>
        object FindControl(string name, Word.Document document)
        {
            try
            {
                foreach (Word.InlineShape shape in document.InlineShapes)
                {
                    if (shape.Type == Word.WdInlineShapeType.wdInlineShapeOLEControlObject)
                    {
                        object oleControl = shape.OLEFormat.Object;
                        Type oleControlType = oleControl.GetType();
                        string oleControlName = (string) oleControlType.InvokeMember("Name", 
                            System.Reflection.BindingFlags.GetProperty, null, oleControl, null);
                        if (String.Compare(oleControlName, name, true, System.Globalization.CultureInfo.InvariantCulture) == 0)
                        {
                            return oleControl;
                        }
                    }
                }

                foreach (Word.Shape shape in document.Shapes)
                {
                    if (shape.Type == Microsoft.Office.Core.MsoShapeType.msoOLEControlObject)
                    {
                        object oleControl = shape.OLEFormat.Object;
                        Type oleControlType = oleControl.GetType();
                        string oleControlName = (string) oleControlType.InvokeMember("Name", 
                            System.Reflection.BindingFlags.GetProperty, null, oleControl, null);
                        if (String.Compare(oleControlName, name, true, System.Globalization.CultureInfo.InvariantCulture) == 0)
                        {
                            return oleControl;
                        }
                    }
                }
            }
            catch
            {
                // Returns null if the control is not found.
            }
            return null;
        }

        #endregion

        /// <summary>
        /// Called when the document is opened.
        /// </summary>
        protected void ThisDocument_Open()
        {
			// Set up command and wire the command bar button event delegates
			// to the Click event handlers.
			if(!SetupCommandBar()) return;
			WireButtonClickEvents();

			FillCustomerList();

			// Set up event handling for closing and saving the document.
			AppDocCloseEvent = 
				new Word.ApplicationEvents4_DocumentBeforeCloseEventHandler(
				App_DocClose);
			ThisApplication.DocumentBeforeClose += AppDocCloseEvent;
			AppDocSaveEvent = new 
				Word.ApplicationEvents4_DocumentBeforeSaveEventHandler(
				App_DocSave);
			ThisApplication.DocumentBeforeSave += AppDocSaveEvent;

			// Add the XML tags to the document and apply formatting to 
			// the document.
			if(!BuildDoc()) return;
			ThisDocument.Saved = true;

        }

		private bool SetupCommandBar()
		{
			try
			{
				// Create a new command bar in the context of this document.
				ThisApplication.CustomizationContext=ThisDocument;
				cbar = ThisApplication.CommandBars.Add("Customer Form", 
					oMissing, oMissing, (object)true);

				// Add a button to the command bar for adding a new customer.
				cbSaveAsNew = (Office.CommandBarButton)(cbar.Controls.Add(
					(object)1, oMissing, oMissing, oMissing, oMissing));
				cbSaveAsNew.Style=Office.MsoButtonStyle.msoButtonCaption;
				cbSaveAsNew.Caption = "Save As New";

				// Add a combo box to the command bar for selecting a customer.
				object objType = Office.MsoControlType.msoControlDropdown;
				cbCustomerList = (Office.CommandBarComboBox)(
					(cbar.Controls.Add(objType, oMissing, oMissing,
					oMissing, (object)true)));
				cbCustomerList.Caption = "Select a Customer:";
				cbCustomerList.Style = Office.MsoComboStyle.msoComboLabel;
				cbCustomerList.ListIndex = 0;
				cbCustomerList.BeginGroup=true;

				// Add a button to the command bar for changing customer
				// data.
				cbEdit = (Office.CommandBarButton)(cbar.Controls.Add(
					(object)1, oMissing, oMissing, oMissing, oMissing));
				cbEdit.Style=Office.MsoButtonStyle.msoButtonCaption;
				cbEdit.Caption = "Edit";

				// Add a button to the command bar for saving changes
				// to customer data; this button is disabled until a
				// customer is selected from the combo box.
				cbSave = (Office.CommandBarButton)(cbar.Controls.Add(
					(object)1, oMissing, oMissing, oMissing, oMissing));
				cbSave.Style=Office.MsoButtonStyle.msoButtonCaption;
				cbSave.Caption = "Save";
				cbSave.Enabled=false;

				// Add a button to the command bar for clearing the customer
				// data on the form.
				cbClear = (Office.CommandBarButton)(cbar.Controls.Add(
					(object)1, oMissing, oMissing, oMissing, oMissing));
				cbClear.Style=Office.MsoButtonStyle.msoButtonCaption;
				cbClear.Caption = "Clear";
				cbClear.BeginGroup = true;

				// Make the new command bar visible.
				cbar.Visible=true;

				return true;
			}
			catch (Exception ex)
			{
				MessageBox.Show("Error creating CommandBar: " + ex.Message, 
					"Customer Form", MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				return false;
			}

		}

		private void cbSaveAsNew_Click(Office.CommandBarButton btn,
			ref bool Cancel)
		{
			try
			{
				if(ValidateCustomer())
				{
					// If no validation errors occurred, try adding the data.
					clsDataAccess da = new clsDataAccess();
					da.UpdateCustomer(clsDataAccess.EditType.Insert,
						ThisDocument.XMLNodes[1].get_XML(true));

					// Disable the AddNew button once the customer has been
					// added and repopulate the Customer list.
					cbSaveAsNew.Enabled=false;
					FillCustomerList();

					// Mark the document as not "dirty."
					cbSaveAsNew.Enabled=false;
					ThisDocument.Saved = true;
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show("Error adding customer data: " + ex.Message, 
					"Customer Form", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}

		}

		private void cbEdit_Click(Office.CommandBarButton btn, ref bool Cancel)
		{
			try
			{
				if(!ClearNodeText()) return;        
        
				// Get a string for the selected customer ID.
				string sCustID = 
					cbCustomerList.get_List(cbCustomerList.ListIndex);

				// Load the data for the selected customer into an
				// XML document.
				clsDataAccess da = new clsDataAccess();
				System.Xml.XmlDocument xmlDoc = new System.Xml.XmlDocument();
				xmlDoc.LoadXml(da.GetCustomerRecord(sCustID));
        
				// Iterate the nodes in the data and populate the XMLNodes
				// in the document.
				string sPrefix = 
					"xmlns:Customer=\"http://tempuri.org/customer.xsd\"";
				object o = 1;
				XmlNodeList nodes = 
					xmlDoc.SelectNodes("/Document/Customers/*");
				foreach (XmlNode node in nodes)
				{        
					string sXPath = String.Format(
						"/Customer:Document/Customer:Customers/Customer:{0}",
						node.Name);
					Word.XMLNode wordNode = ThisDocument.SelectSingleNode(
						sXPath, sPrefix, false);
					wordNode.Range.FormFields.get_Item(ref o).Result = 
						node.InnerText;
				}
                    
				ThisDocument.Saved = true;
				cbSave.Enabled=true;
				cbSaveAsNew.Enabled=false;

				return;
			}
			catch (Exception ex)
			{
				MessageBox.Show("Error modifying customer data: " + ex.Message, 
					"Customer Form", MessageBoxButtons.OK, 
					MessageBoxIcon.Error);
				return;
			}

		}

		private void cbSave_Click(Office.CommandBarButton btn, ref bool Cancel)
		{
			try
			{
				if(ValidateCustomer())
				{
					// If no validation errors occurred, try adding the data.
					clsDataAccess da = new clsDataAccess();
					da.UpdateCustomer(clsDataAccess.EditType.Update,
						ThisDocument.XMLNodes[1].get_XML(true));

					// Mark the document as not "dirty."
					cbSave.Enabled = false;
					ThisDocument.Saved = true;
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show("Error updating customer data: " + ex.Message, 
					"Customer Form", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}

		}

		private void cbClear_Click(Office.CommandBarButton btn, ref bool Cancel)
		{
			ClearNodeText();
			cbSaveAsNew.Enabled=true;
			cbSave.Enabled=false;
			ThisDocument.Saved = true;
			return ;

		}

		private void WireButtonClickEvents()
		{
			// Set up the Click events for the command bar buttons.
			cbSaveAsNewClickEvent = 
				new Office._CommandBarButtonEvents_ClickEventHandler(
				cbSaveAsNew_Click);
			cbSaveAsNew.Click += cbSaveAsNewClickEvent;
			cbEditClickEvent = 
				new Office._CommandBarButtonEvents_ClickEventHandler(
				cbEdit_Click);
			cbEdit.Click += cbEditClickEvent;
			cbSaveClickEvent = 
				new Office._CommandBarButtonEvents_ClickEventHandler(
				cbSave_Click);
			cbSave.Click += cbSaveClickEvent;
			cbClearClickEvent = 
				new Office._CommandBarButtonEvents_ClickEventHandler(
				cbClear_Click);
			cbClear.Click += cbClearClickEvent;
		}

		private void FillCustomerList()
		{
			// Clear the combo box list.
			cbCustomerList.Clear();    

			// Obtain the list of Customer IDs as XML.
			clsDataAccess da = new clsDataAccess();
			StringReader sr = new StringReader(da.GetCustomers());
			XmlTextReader xtr = new XmlTextReader(sr);

			// Read the CustomerID nodes in the XML and then add the
			// values in each node to the combo box list.
			while(xtr.Read())
			{
				if ((xtr.NodeType==XmlNodeType.Element) &
					(xtr.Name=="CustomerID"))
				{
					string sCustomerID = xtr.ReadString();
					cbCustomerList.AddItem(sCustomerID, oMissing);
				}
			}
			xtr.Close();
			sr.Close();
		}

		private bool BuildDoc()
		{
			try
			{
				// Add the Document and Customers nodes.
				Word.XMLNodes nodes = ThisDocument.XMLNodes;

				string ns = "http://tempuri.org/customer.xsd";
				object rng = ThisDocument.Content;
				Word.XMLNode oDSNode = nodes.Add(
					"Document", ns, ref rng);
				rng = oDSNode.Range;
				Word.XMLNode oTblNode = nodes.Add( 
					"Customers", ns, ref rng);

				// Insert an 11x2 table inside the Customers node.
				Word.Table tbl = ThisDocument.Tables.Add(
					oTblNode.Range, 11, 2, ref oMissing, ref oMissing);

				// Format the table.
				Word.Borders brdrs = tbl.Columns[2].Cells.Borders;
				brdrs[Word.WdBorderType.wdBorderHorizontal].LineStyle = 
					Word.WdLineStyle.wdLineStyleSingle;

				// Fill the cells in the table:
				//   Column 1: Label
				//   Column 2: XMLNode and a Form field for fill-in.
				string[] aNames = new string [11]{"CustomerID", "CompanyName",
													 "ContactName", "ContactTitle", "Address", "City", "Region",
													 "PostalCode", "Country", "Phone", "Fax"};
				for(int i=0;i<=10;i++)
				{
					tbl.Cell(i+1,1).Range.Text = aNames[i];
					rng = tbl.Cell(i+1,2).Range;
					Word.XMLNode node = nodes.Add(aNames[i], ns, ref rng);
					ThisDocument.FormFields.Add(
						node.Range, Word.WdFieldType.wdFieldFormTextInput);
				}                
				ThisDocument.FormFields.Shaded=false;

				// Turn on mixed content.
				ThisDocument.XMLSchemaReferences.IgnoreMixedContent = true;

				// Hide table gridlines and protect the document.
				ThisDocument.ActiveWindow.View.TableGridlines = false;
				ThisDocument.Protect(
					Word.WdProtectionType.wdAllowOnlyFormFields, 
					ref oMissing, ref oMissing, ref oMissing, ref oMissing);
        
				return true;
			}
			catch (Exception ex)
			{
				MessageBox.Show("Error building document: " + ex.Message, 
					"Customer Form", MessageBoxButtons.OK,   
					MessageBoxIcon.Error);
				return false;
			}
		}

		private void App_DocClose(Word.Document Doc, ref bool Cancel)
		{
			// Do not proceed if the document closing is not ThisDocument.
			if(!(ThisDocument == Doc)) return;

			// If the document is dirty, determine if the user wants
			// to discard their changes to the Customer data
			// before continuing with the document close.
			if(!ThisDocument.Saved)
			{
				DialogResult ret = MessageBox.Show("You have not " +
					"submitted your changes to the customer data. Close " +
					"this document without submitting your changes?",
					"Customer Form", MessageBoxButtons.YesNo, 
					MessageBoxIcon.Question);
				if(ret==DialogResult.Yes)
				{
					ThisDocument.Saved = true;
				}
				else
				{
					Cancel = true;
				}
			}
			return;
		}

		private void App_DocSave(Word.Document Doc, ref bool SaveAsUI, 
			ref bool Cancel)
		{
			MessageBox.Show("Please save changes using the Customer Form " +
				"CommandBar.", "Customer Form", MessageBoxButtons.OK, 
				MessageBoxIcon.Information);
			Cancel = true;
		}

		private bool ClearNodeText()
		{
			try
			{
				// If the document is dirty, verify that the user 
				// wants to clear the form.
				if(!ThisDocument.Saved)
				{
					DialogResult ret = MessageBox.Show(
						"Discard your current edits?", "Customer Form", 
						MessageBoxButtons.YesNo, MessageBoxIcon.Question);
					if(ret==DialogResult.No) return false;
				}

				// Iterate the child nodes of the Customers node 
				// in the document and set the Result of each 
				// FormField to a null string.
				Word.XMLNodes nodes = ThisDocument.SelectNodes(
					"/Customer:Document/Customer:Customers/*",
					"xmlns:Customer=\"http://tempuri.org/customer.xsd\"", 
					false);        
				object o = 1;
				for(int i=1;i<=nodes.Count;i++)
				{
					Word.XMLNode node = nodes[i];
					node.Range.FormFields.get_Item(ref o).Result = "";
				}
        
				return true;
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message);
				return false;
			}
		}

		private bool ValidateCustomer()
		{
			//Validate the data entered and, if not valid, display the
			//name of the invalid node + the validation error text.
			Word.XMLNodes nodes = ThisDocument.XMLNodes;
			Word.XMLNode node;
			for(int i=1;i<=nodes.Count;i++)
			{
				node = nodes[i];
				node.Validate();
				if (node.ValidationStatus !=
					Word.WdXMLValidationStatus.wdXMLValidationStatusOK)
				{
					MessageBox.Show(node.BaseName + ": " + 
						node.get_ValidationErrorText(false), "Customer Form",
						MessageBoxButtons.OK, MessageBoxIcon.Warning);
					return false;
				}
			}
			return true;
		}

        /// <summary>
        /// Called when the document is closed.
        /// </summary>
        protected void ThisDocument_Close()
        {
            // Ensure that Close is not called twice.
            designModeClose = false;
        }
    }
}
