Imports System.Windows.Forms
Imports Office = Microsoft.Office.Core
Imports Word = Microsoft.Office.Interop.Word
'Imports MSForms = Microsoft.Vbe.Interop.Forms
' Visual Basic
Imports System.Xml
Imports System.Collections

' Office integration attribute. Identifies the startup class for the document. Do not modify.
<Assembly: System.ComponentModel.DescriptionAttribute("OfficeStartupClass, Version=1.0, Class=My_Word_Bookmarks.OfficeCodeBehind")>

Public Class OfficeCodeBehind

    Friend WithEvents ThisDocument As Word.Document
    Friend WithEvents ThisApplication As Word.Application

    ' Determines whether ActiveX controls will be set to
    ' Run mode even if Word is in High security mode.
    ' Set this variable to True if your solution uses
    ' ActiveX controls.
    Private toggleActiveXControls As Boolean = False
    ' Visual Basic
    Private myFolder As String
    Const dataFile As String = "WordReadBookmark.xml"

#Region "Generated initialization code"

    ' Default constructor.
    Public Sub New()
    End Sub

    Private designModeClose As Boolean = False

    ' Required procedure. Do not modify.
    Public Sub _Startup(ByVal application As Object, ByVal document As Object)
        ThisApplication = CType(application, Word.Application)
        ThisDocument = CType(document, Word.Document)

        ' Check if the document is in Design mode.
        If (ThisDocument.FormsDesign = True) Then
            ' Set ActiveX controls to Run mode.
            If (toggleActiveXControls = True) Then
                ThisDocument.ToggleFormsDesign()
            End If

            ' Check if the ThisDocument is a document and not a template.
            If (ThisDocument.Path <> "") Then
                ' Set the shutdown procedure to use design mode and then call the Open handler.
                designModeClose = True
                ThisDocument_Open()

                ' Ensure that Open is not called twice.
                RemoveHandler ThisDocument.Open, AddressOf ThisDocument_Open
            End If
        End If
    End Sub

    ' Required procedure. Do not modify.
    Public Sub _Shutdown()
        ' If the document is in design mode, call the Close handler.
        If (designModeClose = True) Then
            ThisDocument_Close()
        End If

        ThisApplication = Nothing
        ThisDocument = Nothing
    End Sub

    ' Returns the control with the specified name in ThisDocument.
    Overloads Function FindControl(ByVal name As String) As Object
        Return FindControl(name, ThisDocument)
    End Function

    ' Returns the control with the specified name in the specified document.
    Overloads Function FindControl(ByVal name As String, ByVal document As Word.Document) As Object
        Try
            Dim inlineShape As Word.InlineShape
            For Each inlineShape In document.InlineShapes
                If (inlineShape.Type = Word.WdInlineShapeType.wdInlineShapeOLEControlObject) Then
                    Dim oleControl As Object = inlineShape.OLEFormat.Object
                    Dim oleControlType As Type = oleControl.GetType()
                    Dim oleControlName As String = CType(oleControlType.InvokeMember("Name", _
                    Reflection.BindingFlags.GetProperty, Nothing, oleControl, Nothing), String)
                    If (String.Compare(oleControlName, name, True, System.Globalization.CultureInfo.InvariantCulture) = 0) Then
                        Return oleControl
                    End If
                End If
            Next

            Dim shape As Word.Shape
            For Each shape In document.Shapes
                If (shape.Type = Microsoft.Office.Core.MsoShapeType.msoOLEControlObject) Then
                    Dim oleControl As Object = shape.OLEFormat.Object
                    Dim oleControlType As Type = oleControl.GetType()
                    Dim oleControlName As String = CType(oleControlType.InvokeMember("Name", _
                    Reflection.BindingFlags.GetProperty, Nothing, oleControl, Nothing), String)
                    If (String.Compare(oleControlName, name, True, System.Globalization.CultureInfo.InvariantCulture) = 0) Then
                        Return oleControl
                    End If
                End If
            Next

        Catch Ex As Exception
            ' Returns Nothing if the control is not found.
        End Try
        Return Nothing
    End Function
#End Region

    ' Called when a new document is created from the template.
    ' Not called when the document is opened subsequently.
    Private Sub ThisDocument_New() Handles ThisDocument.New
        Dim myTemplate As Word.Template
        myTemplate = CType(ThisDocument.AttachedTemplate(), Word.Template)
        Me.myFolder = myTemplate.Path

        Dim dataFile As String = _
            Me.GetAbsolutePath(OfficeCodeBehind.dataFile)

        Dim doc As XmlDocument = New XmlDocument
        doc.Load(dataFile)

        FillInTheData(doc)
    End Sub

    ' Called when documents that were created from the template are opened,
    ' and when the template is opened for editing.
    ' Not called when a new document is created from the template.
    Private Sub ThisDocument_Open() Handles ThisDocument.Open

    End Sub

    ' Called when documents that were created from the template are closed,
    ' and when the template is closed.
    Private Sub ThisDocument_Close() Handles ThisDocument.Close

        ' Ensure that Close is not called twice.
        designModeClose = False
    End Sub

    ' Visual Basic
    Private Function GetAbsolutePath(ByVal relativePath _
        As String) As String
        Return System.IO.Path.Combine(Me.myFolder, relativePath)
    End Function

    ' Visual Basic
    Public Sub FillInTheData(ByVal doc As XmlDocument)
        Dim bookmarks As Word.Bookmarks = ThisDocument.Bookmarks
        Dim bookmark As Word.Bookmark

        For Each bookmark In bookmarks
            Dim xpath As String = "//" + bookmark.Name
            Dim node As XmlNode = doc.SelectSingleNode(xpath)
            If Not node Is Nothing Then
                bookmark.Range.Text = node.InnerText
            End If
        Next
    End Sub

End Class
